##
##  GUI for v4l device priority tester
##
##  Description:
##
##    This Tcl/Tk script is loaded during start-up of the application.
##    It builds a dialog which shows the current status and allows to
##    trigger tests.
##
##  Author: Tom Zoerner
##
##  $Id: gui.tcl,v 1.3 2005/05/31 20:16:55 tom Exp tom $
##

set dev_path "/dev/video"
set dev_driver_name ""
set prio_levels [list "unset" "background" "interactive" "record"]
set dev_cur_prio [lindex $prio_levels 0]
set dev_cur_freq_norm ""
set dev_prio_app0 1
set dev_prio_app1 1
set app0_test_freq [expr 3140.0/16]
set app0_test_norm "PAL"
set app1_test_freq [expr 2804.0/16]
set app1_test_norm "PAL"

# Device path: [path]
# Device driver: 
# Current device control threshold: bg,interct,record,-failed-
# #1 Test app. device threshold: [radio]
#    Attempt channel change: [button]
#    TV frequency: [n.m MHz]
#    TV norm: [PAL,SECAM,NTSC]
# #2 Test app. device threshold: [select] [button:register]
#    Attempt channel change: [button]
#    TV frequency: [n.m MHz]
#    TV norm: [PAL,SECAM,NTSC]
# Current TV freq. and norm: [text]

proc CreateMainWindow {} {
   global dev_path dev_driver_name
   global dev_cur_prio dev_cur_freq_norm
   global prio_levels

   # create an image of a folder
   set fileImage [image create photo -data {
      R0lGODlhEAAMAKEAAAD//wAAAPD/gAAAACH5BAEAAAAALAAAAAAQAAwAAAIghINhyycvVFsB
      QtmS3rjaH1Hg141WaT5ouprt2HHcUgAAOw==
   }]

   # generate control widgets
   frame  .all
   set row 0

   frame  .all.devpath
   label  .all.devpath.path_lab -text "Device path:"
   grid   .all.devpath.path_lab -row 1 -column 0 -sticky w
   entry  .all.devpath.path_ent -width 34 -textvariable dev_path -exportselection 0
   bind   .all.devpath.path_ent <FocusIn>  {%W selection range 0 end}
   bind   .all.devpath.path_ent <FocusOut> {%W selection clear}
   bind   .all.devpath.path_ent <Return> MountTable_SelChange
   grid   .all.devpath.path_ent -row 1 -column 1 -sticky we
   button .all.devpath.dlgbut -image $fileImage -command {
      set tmp [tk_chooseDirectory -parent .all \
                  -initialdir $mntent_dos \
                  -mustexist 1]
      if {[string length $tmp] > 0} {
         set mntent_dos $tmp
      }
      unset tmp
   }
   grid   .all.devpath.dlgbut -row 1 -column 2
   pack   .all.devpath -side top -fill x -padx 5 -pady 10

   frame  .all.cur
   frame  .all.cur.prio
   label  .all.cur.prio.lab -text "Current device priority level:"
   label  .all.cur.prio.val -textvariable dev_cur_prio
   pack   .all.cur.prio.lab .all.cur.prio.val -side left
   pack   .all.cur.prio -side top -fill x

   frame  .all.cur.freq
   label  .all.cur.freq.lab -text "Current TV freq. & norm:"
   label  .all.cur.freq.val -textvariable dev_cur_freq_norm
   pack   .all.cur.freq.lab .all.cur.freq.val -side left
   pack   .all.cur.freq -side top -fill x

   frame  .all.cur.drv
   label  .all.cur.drv.lab -text "Device driver:"
   label  .all.cur.drv.val -textvariable dev_driver_name
   pack   .all.cur.drv.lab .all.cur.drv.val -side left
   pack   .all.cur.drv -side top -fill x

   frame  .all.cur.status
   scrollbar .all.cur.status.sb -orient vertical -command {.all.cur.status.txt yview}
   pack   .all.cur.status.sb -side left -fill y
   text   .all.cur.status.txt -wrap word -exportselection 0 -heigh 4 -width 1 \
                              -yscrollcommand {.all.cur.status.sb set}
   bindtags .all.cur.status.txt {all .}
   pack   .all.cur.status.txt -side left -fill x -expand 1
   pack   .all.cur.status -side top -fill x
   pack   .all.cur -side top -fill x -padx 5 -pady 10

   for {set app_idx 0} {$app_idx <= 1} {incr app_idx} {
      set frm .all.app{$app_idx}
      frame  ${frm} -borderwidth 2 -relief sunken

      global dev_prio_app${app_idx}
      frame  ${frm}.prio
      label  ${frm}.prio.lab -text "Application #[expr $app_idx + 1] prio:"
      pack   ${frm}.prio.lab -side left -pady 5
      for {set prio_idx 1} {$prio_idx <= 3} {incr prio_idx} {
         radiobutton ${frm}.prio.lev${prio_idx} -text [lindex $prio_levels $prio_idx] \
                                                -command "C_SetPriority $app_idx $prio_idx" \
                                                -variable dev_prio_app${app_idx} -value $prio_idx
         pack ${frm}.prio.lev${prio_idx} -side left -padx 2
      }
      pack ${frm}.prio -side top -fill x

      frame  ${frm}.set_chn
      label  ${frm}.set_chn.lab -text "Attempt channel change:"
      pack   ${frm}.set_chn.lab -side left
      button ${frm}.set_chn.but -text "Test" -command "Cmd_ChangeChannel $app_idx app${app_idx}_test_freq app${app_idx}_test_norm"
      pack   ${frm}.set_chn.but -side left -padx 15 -pady 5
      pack   ${frm}.set_chn -side top -fill x

      frame  ${frm}.tv_freq
      label  ${frm}.tv_freq.lab -text "Test TV freq. & norm:"
      entry  ${frm}.tv_freq.val -textvariable app${app_idx}_test_freq -width 10
      label  ${frm}.tv_freq.lab2 -text "\[MHz\]"
      pack   ${frm}.tv_freq.lab ${frm}.tv_freq.val ${frm}.tv_freq.lab2 -side left
      tk_optionMenu ${frm}.tv_freq.norm app${app_idx}_test_norm "PAL" "SECAM" "NTSC"
      pack   ${frm}.tv_freq.norm -side left -padx 5
      pack   ${frm}.tv_freq -side top -fill x

      pack   ${frm} -side top -fill x -padx 5
   }
   pack   .all -side top

   frame  .cmd
   button .cmd.ok -text "Quit" -width 6 -command {destroy .} -default active
   pack   .cmd.ok -side left -padx 10
   pack   .cmd -side top -pady 10

   # save default font and background colors
   set ::text_bg [.all.devpath.path_ent cget -background]
   set ::frame_bg [.all cget -background]

   option add *Dialog.msg.font {Helvetica -14 bold} userDefault

   wm resizable . 1 0
   update
   wm minsize . [winfo reqwidth .] [winfo reqheight .]

   # install timer to update priority
   after idle QueryTimer
}

##  --------------------------------------------------------------------------
##  Timer function to periodically query device priority
##
set last_dev_path ""

proc QueryTimer {} {
   global dev_path dev_driver_name dev_cur_prio dev_cur_freq_norm
   global dev_prio_app0 dev_prio_app1
   global prio_levels
   global last_dev_path

   #after cancel QueryTimer

   if {[string compare $last_dev_path $dev_path] != 0} {
      .all.cur.status.txt delete 1.0 end
      set ltmp [C_OpenDevice $dev_path]
      set last_dev_path $dev_path
      set dev_driver_name [join $ltmp ","]

      C_SetPriority 0 $dev_prio_app0
      C_SetPriority 1 $dev_prio_app1
   }

   set prio [C_QueryPriority 0]
   if {$prio < [llength $prio_levels]} {
      set dev_cur_prio [lindex $prio_levels $prio]
      set ltmp [C_QueryChannel 0]
      set dev_cur_freq_norm [format "%.2f %s" [expr [lindex $ltmp 0] / 16.0] [lindex $ltmp 1]]
   } else {
      set dev_cur_prio $prio
   }

   after 2000 QueryTimer
}

proc Cmd_ChangeChannel {app_idx var_freq var_norm} {
   global app0_test_freq app1_test_freq
   upvar $var_freq freq
   upvar $var_norm norm

   .all.cur.status.txt delete 1.0 end
   C_ChangeChannel $app_idx [expr int($freq * 16)] $norm
}

##  --------------------------------------------------------------------------
##  Handling of the "About..." dialog
##
set about_popup 0

proc CreateAbout {} {
   global tcl_patchLevel
   global about_popup

   if {$about_popup == 0} {
      CreateTransientPopup .about "About this application"
      wm resizable .about 0 0
      set about_popup 1

      label .about.copyr1 -text "Copyright  2005 by Tom Zoerner\nAll rights reserved."
      label .about.copyr2 -text "Author: tomzo@nefkom.net"
      pack .about.copyr1 -side top -padx 10
      pack .about.copyr2 -side top -padx 10 -pady 10

      message .about.m -aspect 300 -text {

This program contains software licensed under the GNU Public License (aka GPL, see http://www.fsf.org/)  Hence this software may only be redistributed including the full sources.
}
      pack .about.m -side top

      button .about.dismiss -text "Dismiss" -command {destroy .about} -default active
      pack .about.dismiss -pady 10

      bind  .about.dismiss <Destroy> {+ set about_popup 0}
      bind  .about.dismiss <Return> {tkButtonInvoke .about.dismiss}
      bind  .about.dismiss <Escape> {tkButtonInvoke .about.dismiss}
      focus .about.dismiss

   } else {
      raise .about
   }
}


##  --------------------------------------------------------------------------
##  Helper function to modify a font's size or appearance
##
proc DeriveFont {afont delta_size {style {}}} {
   if {[string length $style] == 0} {
      set style [lindex $afont 2]
   }
   set size [lindex $afont 1]
   if {$size < 0} {set delta_size [expr 0 - $delta_size]}
   incr size $delta_size
   
   return [list [lindex $afont 0] $size $style]
}  

##  --------------------------------------------------------------------------
##  Create a popup toplevel window
## 
proc CreateTransientPopup {wname title {parent .}} {
   
   toplevel $wname
   wm title $wname $title
   # make it a slave of the parent, usually without decoration
   wm transient $wname $parent
   # place it in the upper left corner of the parent
   wm geometry $wname "+[expr [winfo rootx $parent] + 30]+[expr [winfo rooty $parent] + 30]"
}

##  --------------------------------------------------------------------------
##  Callback for <Enter> event on entry fields
##  - select the entire text in the widget so that the user needs not select
##    and clear the old text, but can just start typing
##  - only useful for small fields where the old content is frequently discarded
##
proc SelectTextOnFocus {wid} {
   if {[string compare [focus -displayof $wid] $wid] != 0} {
      $wid selection range 0 end
      focus $wid
   }
}

